%% gh -- a compiler for Generic Haskell.
%% Copyright (c) 2001  The Generic Haskell Team. Utrecht University
%% 
%% This library is free software; you can redistribute it and/or modify
%% it under the terms of the GNU Lesser General Public License as
%% published by the Free Software Foundation; either version 2.1 of the
%% License, or (at your option) any later version.
%% 
%% This library is distributed in the hope that it will be useful, but
%% WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%% Lesser General Public License for more details.
%% 
%% You should have received a copy of the GNU Lesser General Public
%% License along with this library; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
%%
%% $Id: ErrorUtils.lhs,v 1.1 2001/11/01 11:50:18 cvs-4 Exp $
%%
%% author: Andres Loeh (andres@cs.uu.nl)

%***************************************************************************
\section{Error message utilities}
%***************************************************************************

\begin{code}
module ErrorUtils
  ( internalError, unstaticError, errorLayout, Exc(..), fromOK, notused )
  where

import Utils
import Name
\end{code}

The function |internalError| is supposed to be called in occasions
where a resulting error is a programming error of the GH prototype and
should not just be caused by specific user input.

\begin{code}
internalError                  :: String -> a
internalError msg              =  error $ nl
                               ++ errorLayout (pre ++ msg ++ post)
  where
  pre  = "INTERNAL ERROR\ndescription: "
  post = "\n\nYou might want to issue a bug report.\n"
      ++ "Please send an input file that caused the error together with\n"
      ++ "the error description to " ++ email ++ ".\n"
      ++ "Sorry for the inconvenience.\n"

unstaticError                  :: String -> a
unstaticError msg              =  error $ nl
                               ++ errorLayout (pre ++ msg ++ post)
  where
  pre  = "ERROR (should have been caught): "
  post = ""

errorLayout                    :: String -> String
errorLayout                    =  unlines
                               .  prefix
                               .  lines
  where prefix (x:xs)          =  ("*** " ++ x) : map prefix' xs
        prefix' x              =  " |  " ++ x

data Exc a = OK a | Error

instance Monad Exc where
  return  = OK
  m >>= f = case m of
              Error    -> Error
              OK x     -> f x

fromOK (OK x) = x
\end{code}

An error messages used to provide input attributes that should not be 
needed.

\begin{code}
notused                         :: a
notused                         =  internalError "Illegal use of input attribute."
\end{code}

