%% gh -- a compiler for Generic Haskell.
%% Copyright (c) 2001  The Generic Haskell Team. Utrecht University
%% 
%% This library is free software; you can redistribute it and/or modify
%% it under the terms of the GNU Lesser General Public License as
%% published by the Free Software Foundation; either version 2.1 of the
%% License, or (at your option) any later version.
%% 
%% This library is distributed in the hope that it will be useful, but
%% WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%% Lesser General Public License for more details.
%% 
%% You should have received a copy of the GNU Lesser General Public
%% License along with this library; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
%%
%% $Id: GHILexer.lhs,v 1.3 2002/06/26 17:26:35 cvs-4 Exp $
%%
%% author: Andres Loeh (andres@cs.uu.nl)
%% based on: /hslibs/hssource/HsLexer by Simon Marlow and Sven Panne

(modified version of GHLexer; maybe modularise later)

> module GHILexer (Token(..), get, isSymbol) where
> 
> import GHParseMonad
> import GHIParseUtils
> import AbstractD
> --import FiniteMap as FM
> 
> import Char
> import Maybe

> data Token
>         = VarId String
>         | StringTok String Range  -- { } bounded string
> 
> -- Symbols
> 
> 	  | LeftParen
> 	  | RightParen
> 	  | SemiColon
> 
> -- Reserved operators
>
> 	| Equals
>       | DoubleColon
> 
> -- Reserved Ids
> 
>       | KW_Module
>       | KW_ExportTp
>       | KW_ExportVal
>       | KW_SetModule
>       | KW_Import
>       | KW_Consig
>       | KW_Abstract
>       | KW_Binding
>       | KW_Data
>       | KW_Decl
> 	| KW_Id
> 	| KW_Sym
> 	| KW_Spec   
> 	| KW_Sum     
> 	| KW_Con
>       | KW_Label
> 	| KW_Prod
> 	| KW_Unit     
>       | KW_Fun
> 	| KW_Type
>       | KW_Kind
>       | KW_Strtype
>       | KW_Pkt
>       | KW_Pts     
>       | EOF
>         deriving (Eq,Show)
> 
> reserved_ops :: [(String, Token)] -- FM.FiniteMap Char Token
> -- reserved_ops = FM.fromList [
> reserved_ops = [
>  ( "=",  Equals )
>  ]
> 
> reserved_ids :: [(String, Token)] -- FM.FiniteMap Char Token
> -- reserved_ids = FM.fromList [
> reserved_ids = [
>  ( "module",    KW_Module ),
>  ( "export_type",  KW_ExportTp ),
>  ( "export_value", KW_ExportVal ),
>  ( "set_module",   KW_SetModule ),
>  ( "import",    KW_Import ),
>  ( "consig",    KW_Consig ),
>  ( "abstract",  KW_Abstract ),
>  ( "binding",   KW_Binding ),
>  ( "data",      KW_Data ),
>  ( "decl",      KW_Decl ),
>  ( "id",        KW_Id ),
>  ( "sym",       KW_Sym ),
>  ( "spec",      KW_Spec ),
>  ( "sum",       KW_Sum ),
>  ( "con",       KW_Con ),
>  ( "label",     KW_Label ),
>  ( "prod",      KW_Prod ),
>  ( "unit",      KW_Unit ),
>  ( "type",      KW_Type ),
>  ( "kind",      KW_Kind ),
>  ( "strtype",   KW_Strtype ),
>  ( "pkt",       KW_Pkt ),
>  ( "pts",       KW_Pts )
>  ]
> 
> isIdent  c = isAlpha c || isDigit c || c == '\'' || c == '_'
> isSymbol c = elem c ":!#$%&*+./<=>?@\\^|-~"
> isWhite  c = elem c " \n\r\t\v\f"
> 
> tAB_LENGTH = 8 :: Int

The source location, (y,x), is the coordinates of the previous token.
col is the current column in the source file.  If col is 0, we are
somewhere at the beginning of the line before the first token.

Setting col to 0 is used in two places: just after emitting a virtual
close brace due to layout, so that next time through we check whether
we also need to emit a semi-colon, and at the beginning of the file,
to kick off the lexer.

> get :: Lex Token
> get =  Lex lexer
> 
> lexer :: (Token -> P a) -> P a
> lexer cont input (Position_Position f y x) col =
>          if col == 0
>             then tab y x True  input
>             else tab y col False input -- throw away old x
>   where
>    	-- move past whitespace and comments
>         tab y x bol [] = 
>         	cont EOF [] (Position_Position f y x) col
>         tab y x bol ('\t':s) =
>         	tab y (nextTab x) bol s
>         tab y x bol ('\n':s) =
>                 newLine cont s y
>         tab y x bol ('-':'-':s) = 
>         	newLine cont (drop 1 (dropWhile (/= '\n') s)) y
>         tab y x bol ('{':'-':s) = nestedComment tab y x bol s
>         tab y x bol (c:s)
>         	| isWhite c = tab y (x+1) bol s
>         	| otherwise = lexToken cont (c:s) (Position_Position f y x) x
> 
> 	  newLine cont s y =  tab (y+1) 1 True s
> 
> nextTab x = x + (tAB_LENGTH - (x-1) `mod` tAB_LENGTH)
>
> lexToken :: (Token -> P a) -> P a 
> lexToken cont (c:s) loc@(Position_Position f y x') x =
>    -- trace ("lexer: y="++show y++" x="++show x++"\n") $ 
>    case c of
>         -- First the special symbols
>         '(' -> special LeftParen
>         ')' -> special RightParen
>         ';' -> special SemiColon
>         '=' -> special Equals
>         '{' -> lexString cont s loc (x+1)
> 
>         c | isLower c || c == '_' ->
>         	let 
>         	    (idtail, rest) = span isIdent s
>         	    id = c:idtail
>         	    l_id = 1 + length idtail
>         	in
>         	case lookup id reserved_ids of
>         		Just keyword -> forward l_id keyword rest
>         		Nothing -> forward l_id (VarId id) rest 
> 
>           | otherwise ->
>         	\ stk -> fail ("illegal character \'" ++ show c ++ "\'\n")
> 
>  where special t = forward 1 t s
>        forward n t s = cont t s loc (x+n)
> 
> lexToken _ _ _ _ = error "Internal error: empty input in lexToken"
>
> lexString :: (Token -> P a) -> P a
> lexString cont s loc@(Position_Position f y' _) x' = loop 0 "" s x' y'
>   where
>      loop l e s x y = case s of
>             '\\':'{':s  -> loop l ("\\{" ++ e) s (x+2) y
>             '\\':'}':s  -> loop l ("\\}" ++ e) s (x+2) y
>             '\n':s      -> loop l ('\n':e) s 1 (y+1)
>             '{':s       -> loop (l+1) ('{':e) s (x+1) y
>             '}':s | l == 0 -> cont (StringTok (reverse e) (Range_Range (Position_Position f y' x')
>                                                                        (Position_Position f y  x )))
>                                                                        s (Position_Position f y x) (x+1)
>                   | otherwise -> loop (l-1) ('}':e) s (x+1) y
>             c:s		-> loop l (c:e) s (x+1) y
>             []          -> \ stk -> fail "Improperly terminated string"
> 
> nestedComment cont y x bol s =
>    case s of
>       '-':'}':s -> cont y (x+2) bol s
>       '{':'-':s -> nestedComment (nestedComment cont) y (x+2) bol s
>       '\t':s    -> nestedComment cont y (nextTab x) bol s
>       '\n':s    -> nestedComment cont (y+1) 1 True s
>       c:s       -> nestedComment cont y (x+1) bol s
>       []        -> error "Internal error: nestedComment"
