%% gh -- a compiler for Generic Haskell.
%% Copyright (c) 2001  The Generic Haskell Team. Utrecht University
%% 
%% This library is free software; you can redistribute it and/or modify
%% it under the terms of the GNU Lesser General Public License as
%% published by the Free Software Foundation; either version 2.1 of the
%% License, or (at your option) any later version.
%% 
%% This library is distributed in the hope that it will be useful, but
%% WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%% Lesser General Public License for more details.
%% 
%% You should have received a copy of the GNU Lesser General Public
%% License along with this library; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
%%
%% $Id: GHParseMonad.lhs,v 1.2 2002/04/15 11:56:00 cvs-4 Exp $
%%
%% author: andres@cs.uu.nl
%% based on: /hslibs/hssource/HsParseMonad by Simon Marlow and Sven Panne;
%%           large parts are copied unchanged

> module GHParseMonad           (  Result(..), LexContext(..), P, Lex(..), unLex, run
>                               ,  parseError, getPosition, getContext, pushContext, popContext, layoutOn)
> where
>
> import AbstractD -- should be replaced by SrcLoc
> import ErrorUtils
> import Monad
> -- import IOExts

A simple exception monad.

> data Result a                 =  Fail String | Return a
>                                  deriving (Show)

> instance Monad Result where
>     Fail s   >>= _k           =  Fail s
>     Return a >>= k            =  k a
>     return                    =  Return
>     fail                      =  Fail

> instance MonadPlus Result where
>     mzero                     =  fail ""
>     Fail _s `mplus` m         =  m
>     Return a `mplus` _m       =  Return a

> data LexContext               =  NoLayout | Layout Int
> 	                           deriving (Eq, Ord, Show)
>
> type Contexts                 =  [LexContext]

A state monad (in CPS style).

> type P a
>     =  String                 -- input string
>     -> Position               -- location of last token read
>     -> Int                    -- current column
>     -> Contexts               -- layout information
>     -> Result a

> newtype Lex a                 =  Lex (forall ans . (a -> P ans) -> P ans)
> unLex (Lex x)                 =  x

> run                           :: Lex a -> P a
> run (Lex p)                   =  p (\a -> \i l c s -> Return a)

> instance Monad Lex where
>     return a                  =  Lex (\ cont -> cont a)
>     m >>= k                   =  Lex (\ cont -> unLex m (\ a -> unLex (k a) cont))
>     fail s                    =  Lex (\ cont i l c stk -> fail s)

> parseError                    :: String -> Lex a
> parseError s                  =  Lex (\ cont r (Position_Position f y x) c stk -> 
>                                         fail (txt f y x))
>   where txt f y x = errorLayout 
>                 $ f ++ ":" ++ show y ++ ":" ++ show x ++ " ERROR: SYNTAX ERROR\n" 
>                ++ s

Accessing the state.

> getPosition                   :: Lex Position
> getPosition                   =  Lex (\cont i l c s -> cont l i l c s)

> getContext                    :: Lex Contexts
> getContext                    =  Lex (\cont i l c s -> cont s i l c s)

> setBOL                        :: Lex ()
> setBOL                        =  Lex (\cont i l c s -> cont () i l 0 s)

> pushContext                   :: LexContext -> Lex ()
> pushContext ctxt              = 
>     Lex (\cont i l c s -> cont () i l c (ctxt:s))

> popContext                    :: Lex ()
> popContext                    =
>     Lex (\ cont i l@(Position_Position f y x) c stk ->
>            case stk of
>                []      -> fail (f ++ ":" ++ show y ++ ":" ++ show x ++ ": " ++ "Internal error: empty context in popContext")
>   	         (_ : s) -> cont () i l c s)

> layoutOn                      :: Bool -> Lex ()
> layoutOn strict               =  {-
>     getPosition >>= \(Position_Position _ r c) -> trace ("layout on " ++ show c) $ pushContext (Layout c)
> -}
>     do loc@(Position_Position f y x) <- getPosition
>        stk <- getContext
>        case stk of
>          Layout prev_off : _
>            | if strict then prev_off >= x else prev_off > x
>                 -> do -- trace ("layout hack " ++ show loc ++ " (versus " ++ show prev_off ++ ")") $ return ()
>                       pushContext (Layout (x + 1))
>                       setBOL
>          _      -> do -- trace ("layout on " ++ show loc) $ return ()
>                       pushContext (Layout x)

