%% gh -- a compiler for Generic Haskell.
%% Copyright (c) 2001  The Generic Haskell Team. Utrecht University
%% 
%% This library is free software; you can redistribute it and/or modify
%% it under the terms of the GNU Lesser General Public License as
%% published by the Free Software Foundation; either version 2.1 of the
%% License, or (at your option) any later version.
%% 
%% This library is distributed in the hope that it will be useful, but
%% WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%% Lesser General Public License for more details.
%% 
%% You should have received a copy of the GNU Lesser General Public
%% License along with this library; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
%%
%% $Id: GHParseUtils.lhs,v 1.2 2002/04/15 11:56:00 cvs-4 Exp $
%%
%% author: andres@cs.uu.nl
%% based on: /hslibs/hssource/HsParseUtils by Simon Marlow and Sven Panne;
%%           large parts are copied unchanged

ToDo: Polish readInteger, readRational

> module GHParseUtils (
> 	  splitTyConApp		-- Type -> Lex (QualName,[Type])
> 	, mkRecConstrOrUpdate	-- HsExp -> [HsFieldUpdate] -> Lex HsExp
> 	, checkPrec 		-- String -> Lex String
> 	, checkContext		-- Type -> Lex Context
> 	, checkAssertion	-- Type -> Lex Class
> 	, checkPattern		-- HsExp -> Lex HsPat
> 	, checkPatterns		-- [HsExp] -> Lex [HsPat]
> 	, checkExpr		-- HsExp -> Lex HsExp
> 	, checkValDef		-- (SrcLoc, HsExp, HsRhs, [HsDecl]) -> Lex HsDecl
>       , checkSingle           -- SDecl -> Lex SDecl
>       , checkSType            -- Type -> Lex SType
>       , checkInstance         -- QualType -> Lex (Context,QualName,Types)
>       , checkSimpletype       -- QualType -> Lex (Context,QualName,Args)
> 	, checkUnQual		-- HsQName -> Lex HsName
> 	, readInteger 		-- String -> Integer
> 	, readRational 		-- String -> Rational
>       , checkStar             -- QualName -> Lex ()
>       , groupDecls
>       , groupSDecls
>  ) where
> 
> import AbstractD
> import GHParseMonad
> import ErrorUtils
> import Name
> 
> import Char(isDigit,isOctDigit,isHexDigit,digitToInt)
> import Ratio

Splits a type application into the name of the type constructor and 
the list of its arguments. (ks)

> splitTyConApp :: Type -> Lex (QualName,[Type])
> splitTyConApp t = split t []
>  where
> 	split :: Type -> [Type] -> Lex (QualName,[Type])
> 	split (Type_App t u) ts = split t (u:ts)
> 	split (Type_Con _ t) ts = return (t,ts)
> 	split _ _ = parseError "Illegal data/newtype declaration"

Various Syntactic Checks.

In particular, in |checkContext| is tested if a type can be read as a
context for a qualified type. For this, it has to be a tuple of assertions
or a single assertion. Assertions are applications of type constructors to
something.

> checkContext :: Type -> Lex Context
> checkContext tp | isTuple 0 tp = checkTupleContext tp >>= \cs ->
>                                  return (Context_Context (reverse cs))
>                 | otherwise    = checkAssertion tp >>= \c ->
>                                  return (Context_Context [c])
>   where isTuple :: Int -> Type -> Bool
>         isTuple n (Type_App l r) = isTuple (n+1) l
>         isTuple n (Type_Tuple n') = n == n'
>         isTuple _ _ = False

> checkTupleContext :: Type -> Lex Classes
> checkTupleContext (Type_App l r) = checkAssertion r >>= \c ->
>                                    checkTupleContext l >>= \cs ->
>                                    return (c:cs)
> checkTupleContext (Type_Tuple _) = return []
> checkTupleContext _              = internalError "isTuple malfunction"

Changed for multi-parameter type classes.

> checkAssertion :: Type -> Lex Class
> checkAssertion = checkAssertion' []
> 	where	checkAssertion' ts (Type_Con _ c) = return (Class_Class c ts)
> 		checkAssertion' ts (Type_App a t) = checkAssertion' (t:ts) a
> 		checkAssertion' _ _ = parseError "Illegal class assertion"

[As far as I can see this is no longer needed.]
 
< checkDataHeader :: HsQualType -> Lex (HsContext,HsName,[HsName])
< checkDataHeader (HsQualType cs t) =
<    checkSimple t []	     >>= \(c,ts) ->
<    return (cs,c,ts)
< checkDataHeader (HsUnQualType t) =
<    checkSimple t []	     >>= \(c,ts) ->
<    return ([],c,ts)

< checkSimple :: HsType -> [HsName] -> Lex ((HsName,[HsName]))
< checkSimple (HsTyApp l (HsTyVar a)) xs = checkSimple l (a:xs)
< checkSimple (HsTyCon (UnQual t))    xs = return (t,xs)
< checkSimple _ _ = parseError "Illegal data/newtype declaration"


Checking Patterns.

We parse patterns as expressions and check for valid patterns below,
converting the expression into a pattern at the same time.

> checkPattern :: Expr -> Lex Pattern
> checkPattern e = checkPat e []
> 
> checkPatterns :: Exprs -> Lex Patterns
> checkPatterns es = mapM checkPattern es
> 
> checkPat :: Expr -> Patterns -> Lex Pattern
> checkPat (Expr_Con _ c) args = return (Pattern_Con c args)
> checkPat (Expr_App f x) args = checkPat x [] >>= \x -> checkPat f (x:args)
> checkPat e [] = case e of
> 	Expr_Var _ n@(QualName_Plain _ x)
>                          -> return (Pattern_Var n)
> 	Expr_Lit l         -> return (Pattern_Lit l)
> 	Expr_OpApp op l r  -> checkPat l [] >>= \l ->
> 			      checkPat r [] >>= \r ->
>                             return (Pattern_OpApp op l r)
> 	Expr_Tuple es      -> mapM (\e -> checkPat e []) es >>= \ps ->
> 			      return (Pattern_Tuple ps)
> 	Expr_List es	   -> mapM (\e -> checkPat e []) es >>= \ps ->
> 			      return (Pattern_List ps)
> 	Expr_Parens e	   -> checkPat e [] >>= (return . Pattern_Parens)
> 	Expr_As n e	   -> checkPat e [] >>= (return . Pattern_As n)
> 	Expr_Wild          -> return Pattern_Wild
> 	Expr_Irr e	   -> checkPat e [] >>= (return . Pattern_Irr)
> 	Expr_LabCon c fs   -> mapM checkPatField fs >>= \fs ->
> 			      return (Pattern_Lab c fs)
> 	Expr_Negate (Expr_Lit l) 
>                          -> return (Pattern_Negate (Pattern_Lit l))
> 	_ -> patFail
> 
> checkPat _ _ = patFail
> 
> checkPatField :: FBind -> Lex PBind
> checkPatField (FBind_FBind n e) = 
>    checkPat e [] >>= \p ->return (PBind_PBind n p)
> 
> patFail = parseError "Illegal pattern"


Check Expression Syntax.
This is basically to ensure that there are no pattern-syntax
occurences in the expressions (ks).

> checkExpr :: Expr -> Lex Expr
> checkExpr e = case e of
> 	Expr_Var _ _	          -> return e
>       Expr_Con _ _              -> return e
> 	Expr_Lit _                -> return e
> 	Expr_OpApp op l r         -> check2Exprs l r (Expr_OpApp op)
> 	Expr_App l r		  -> check2Exprs l r (Expr_App)
> 	Expr_Negate e             -> check1Expr e Expr_Negate
> 	Expr_Lam ps e             -> check1Expr e (Expr_Lam ps)
> 	Expr_Let bs e             -> check1Expr e (Expr_Let bs)
> 	Expr_If c t e             -> check3Exprs c t e Expr_If
> 	Expr_Case e alts	  -> mapM checkAlt alts >>= \alts ->
> 				     checkExpr e >>= \e ->
> 				     return (Expr_Case e alts)
>       Expr_TyApp _ _ _          -> return e
> 	Expr_Do stmts		  -> mapM checkStmt stmts >>= (return . Expr_Do)
> 	Expr_Tuple es		  -> checkManyExprs es Expr_Tuple
> 	Expr_List es		  -> checkManyExprs es Expr_List
> 	Expr_Parens e		  -> check1Expr e Expr_Parens
> 	Expr_LeftSection op e	  -> check1Expr e (Expr_LeftSection op)
> 	Expr_RightSection op e    -> check1Expr e (Expr_RightSection op)
> 	Expr_LabCon c fields	  -> mapM checkField fields >>= \fields ->
> 				     return (Expr_LabCon c fields)
> 	Expr_LabUp e fields	  -> mapM checkField fields >>= \fields ->
> 				     checkExpr e >>= \e ->
> 				     return (Expr_LabUp e fields)
> 	Expr_EnumFrom e		  -> check1Expr e Expr_EnumFrom
> 	Expr_EnumFromTo e1 e2	  -> check2Exprs e1 e2 Expr_EnumFromTo
> 	Expr_EnumFromThen e1 e2   -> check2Exprs e1 e2 Expr_EnumFromThen
> 	Expr_EnumFromThenTo e1 e2 e3 -> check3Exprs e1 e2 e3 Expr_EnumFromThenTo
> 	Expr_Compr e stmts        -> mapM checkQual stmts >>= \stmts ->
> 				     checkExpr e >>= \e ->
> 				     return (Expr_Compr e stmts)
> 	Expr_TypeSig e ty         -> checkExpr e >>= \e ->
> 				     return (Expr_TypeSig e ty)
> 	_                         -> parseError "Pattern syntax in expression"

Type signature for polymorphic recursion!!

> check1Expr :: Expr -> (Expr -> a) -> Lex a
> check1Expr e f = checkExpr e >>= (return . f)
> 
> check2Exprs :: Expr -> Expr -> (Expr -> Expr -> a) -> Lex a
> check2Exprs e1 e2 f = 
> 	checkExpr e1 >>= \e1 ->
> 	checkExpr e2 >>= \e2 ->
> 	return (f e1 e2)
> 
> check3Exprs :: Expr -> Expr -> Expr -> (Expr -> Expr -> Expr -> a) -> Lex a
> check3Exprs e1 e2 e3 f = 
> 	checkExpr e1 >>= \e1 ->
> 	checkExpr e2 >>= \e2 ->
> 	checkExpr e3 >>= \e3 ->
> 	return (f e1 e2 e3)
> 
> checkManyExprs es f =
> 	mapM checkExpr es >>= \es ->
> 	return (f es) 
> 
> checkAlt (Alt_Alt {- loc -} p galts bs) 
> 	= checkCase galts >>= \galts -> return (Alt_Alt {- loc -} p galts bs)
> 
> checkCase (Case_Unguarded e) = check1Expr e Case_Unguarded
> checkCase (Case_Guarded galts) 
>     	= mapM checkGAlt galts >>= (return . Case_Guarded)
> 
> checkGAlt (GuardedAlt_GuardedAlt {- loc -} e1 e2) 
>       = check2Exprs e1 e2 (GuardedAlt_GuardedAlt {- loc -})
> 
> checkQual (Qual_Gen p e)    = check1Expr e (Qual_Gen p)
> checkQual (Qual_Guard e)    = check1Expr e Qual_Guard
> checkQual s@(Qual_Let bs)   = return s
> 
> checkStmt (Stmt_Gen p e)    = check1Expr e (Stmt_Gen p)
> checkStmt (Stmt_Expr e)     = check1Expr e Stmt_Expr
> checkStmt s@(Stmt_Let bs)   = return s
> 
> checkField (FBind_FBind n e) = check1Expr e (FBind_FBind n)

Check Equation Syntax.
This distinguishes pattern from normal function bindings.

> checkValDef :: (Position, Expr, Rhs, SDecls) -> Lex SDecl
> checkValDef (srcloc, lhs, rhs, whereBinds) =
>     case (isFunLhs lhs []) of
>        (Just (f,Just tp,es)) -> checkPatterns es >>= \ps ->
>                                 checkSType False tp >>= \t ->
>                                 return (SDecl_PolyBinding srcloc f t
>                                   [Binding_Binding ps rhs whereBinds])
>  	 (Just (f,Nothing,es)) -> checkPatterns es >>= \ps ->
> 	          	          return (SDecl_Binding srcloc f 
> 			            [Binding_Binding ps rhs whereBinds])
>        (Nothing)             -> checkPattern lhs >>= \lhs ->
> 		  	          return (SDecl_PatBinding srcloc lhs rhs whereBinds)

Check function bindings.
Polytypic and pattern binding are disallowed in some positions.

> checkSingle :: SDecl -> Lex SDecl
> checkSingle (SDecl_PolyBinding _ _ _ _)= parseError "Generic function binding disallowed here."
> checkSingle (SDecl_PatBinding _ _ _ _) = parseError "Pattern binding disallowed here."
> checkSingle x                          = return x

Check structure types (preliminary).

> checkSType :: Bool {- true if type indexed type -}
>            -> Type -> Lex SType
> checkSType _ (Type_Con _ (QualName_Plain _ (Name_Symbol _ ":+:"))) = return SType_Sum
> checkSType _ (Type_Con _ (QualName_Plain _ (Name_Symbol _ ":*:"))) = return SType_Prod
> checkSType _ (Type_Con _ (QualName_Plain _ (Name_Ident _ "Unit"))) = return SType_Unit
> checkSType _ (Type_Con _ (QualName_Plain _ (Name_Ident _ "Sum"))) = return SType_Sum
> checkSType _ (Type_Con _ (QualName_Plain _ (Name_Ident _ "Prod"))) = return SType_Prod
> checkSType _ (Type_Con _ (QualName_Plain _ (Name_Ident _ "Fun"))) = return SType_Fun
> checkSType True (Type_Con _ (QualName_Plain _ (Name_Ident _ "Con"))) = return (SType_Con $ impossibleName)
> checkSType True (Type_Con _ (QualName_Plain _ (Name_Ident _ "Label"))) = return (SType_Label $ impossibleName)
> checkSType False (Type_App (Type_Con _ (QualName_Plain _ (Name_Ident _ "Con")))
>                            (Type_Var _ var)) = return (SType_Con var)
> checkSType False (Type_App (Type_Con _ (QualName_Plain _ (Name_Ident _ "Label")))
>                            (Type_Var _ var)) = return (SType_Label var)
> checkSType _ Type_Fun = return SType_Fun
> checkSType _ Type_Unit = return (SType_Const genloc hsUnitName)
> checkSType _ Type_List = return (SType_Const genloc hsListName) 
> checkSType _ (Type_Tuple n) = return (SType_Const genloc (hsTupleName n))
> checkSType _ (Type_Con sl qn)  = return (SType_Const sl qn)
> checkSType _ (Type_Var sl var) = return (SType_Var sl var)
> checkSType _ (Type_ConCase sl qn) = return (SType_ConCase sl qn)
> checkSType _ _ = parseError "Malformed type argument in generic declaration"

A variable binding is parsed as an HsPatBind.
Therefore we try to recover a function binding from expression
syntax at this point. Returned is a pair of the function name, and
the list of arguments, if successful.

Warning (ks): We view value bindings as function bindings rather than
pattern bindings.

> isFunLhs (Expr_OpApp op l r) es            = Just (op,Nothing,l:r:es)
> -- isFunLhs (Expr_App (Expr_Var _ f) e) es      = Just (f,e:es)
> isFunLhs (Expr_App (Expr_Parens f) e) es   = isFunLhs f (e:es)
> isFunLhs (Expr_App f e) es                 = isFunLhs f (e:es)
> isFunLhs (Expr_Var _ f) es                 = Just (f,Nothing,es)
> isFunLhs (Expr_TyApp _ f tp) es            = Just (f,Just tp,es)
> isFunLhs _ _                               = Nothing

Convert a type into an instance declaration. 

> checkInstance :: QualType -> Lex (Context,QualName,Types)
> checkInstance (QualType_QualType ctx tp) = 
>     do  (n,tps) <- flattenType tp []
>         return (ctx,n,tps)
>   where flattenType :: Type -> Types -> Lex (QualName,Types)
>         flattenType (Type_App l r) args = flattenType l (r:args)
>         flattenType (Type_Con _ n) args = return (n,args)
>         flattenType _ _ = parseError "Malformed instance declaration"

Dissect a type. This is similar to the case for instances, but
we have to additionally check whether the arguments are all type
variables.

> checkSimpletype :: QualType -> Lex (Context,QualName,Args)
> checkSimpletype tp = do (ctx,qn,tps) <- checkInstance tp
>                         checkUnQual qn
>                         args <- mapM checkArg tps
>                         return (ctx,qn,args)
>   where checkArg :: Type -> Lex QualName
>         checkArg (Type_Var _ n) = return n
>         checkArg _ = parseError "Illegal type argument -- variable required"

Check that an identifier or symbol is unqualified. For occasions when
doing this in the grammar would cause conflicts.

> checkUnQual :: QualName -> Lex Name
> checkUnQual (QualName_Qual _ _ _)   = parseError "Illegal qualified name"
> checkUnQual (QualName_Plain _ n)    = return n

Miscellaneous utilities.

> checkPrec :: String -> Lex String
> checkPrec cs@[c] | isDigit c = return cs
> checkPrec cs                 = parseError ("Illegal precedence " ++ cs)

Stolen from Hugs' Prelude.

> readInteger :: String -> Integer
> readInteger ('0':'o':ds) = readInteger2  8 isOctDigit ds
> readInteger ('0':'x':ds) = readInteger2 16 isHexDigit ds
> readInteger          ds  = readInteger2 10 isDigit    ds
> 
> readInteger2 :: Integer -> (Char -> Bool) -> String -> Integer
> readInteger2 radix isDig ds 
>   = foldl1 (\n d -> n * radix + d) (map (fromIntegral . digitToInt) ds)

Hack ...

> readRational :: String -> Rational
> readRational xs = (readInteger (i++m))%1 * 10^^(case e of {[] -> 0;  ('+':e2) -> read e2; _ -> read e} - length m)
>   where (i,r1) = span isDigit xs
>         (m,r2) = span isDigit (dropWhile (=='.') r1)
>         e      = dropWhile (=='e') r2
>
 
< mkModule 
<   	:: Module 
< 	-> Maybe [HsExportSpec] 
< 	-> ([HsImportDecl],[HsDecl]) 
< 	-> HsModule
< 
< mkModule m e (imp, d) 
<    = HsModule m e (reverse imp) (reverse d)
 
Distinguish between record constructions and updated (ks).

> mkRecConstrOrUpdate :: Expr -> FBinds -> Lex Expr
> mkRecConstrOrUpdate (Expr_Con _ c) fs = return (Expr_LabCon c   fs)
> mkRecConstrOrUpdate exp    fs@(_:_)   = return (Expr_LabUp  exp fs)
> mkRecConstrOrUpdate _      _          = 
>    parseError "Empty record update"

Checking kind syntax.

> checkStar :: QualName -> Lex ()
> checkStar (QualName_Plain _ (Name_Symbol _ "*")) = return ()
> checkStar _                                      = parseError "Illegal kind syntax; * expected"

Grouping declarations.

Sometime later I should think about how to rewrite this function using
higher order functions. Essentially, this is nothing more than a 
|foldr01|.

> groupDecls                    ::  [Decl] -> [Decl]
> groupDecls []                 =   []
> groupDecls [s]                =   [s]
> groupDecls (s1 : ss)          =   
>   let
>     (s2 : r) = groupDecls ss
>     f = case (s1,s2) of {

This is the case of polykinded types. We can have multiple
declarations for one polykinded type, but they must be next to each
other. We compare the name of the declared type.

>           (Decl_Top (TDecl_Poly srcloc n1 b )
>           ,Decl_Top (TDecl_Poly _      n2 bs)
>           )        | n1 == n2 -> [Decl_Top 
>                                      (TDecl_Poly srcloc n1 (b ++ bs))]

Now ordinary function definitions. These are also grouped by name
equality.

>         ; (Decl_Simple (SDecl_Binding srcloc n1 b )
>           ,Decl_Simple (SDecl_Binding _      n2 bs)
>           )        | n1 == n2 -> [Decl_Simple
>                                      (SDecl_Binding srcloc n1 (b ++ bs))]

And finally generic function definitions. Here, the same name is not
enough because lines for different constructors go into different
component functions later. But we may have more than one declaration
for one function and one constructor, especially in the case of the
|+|. Therefore, we group these.

>         ; (Decl_Simple (SDecl_PolyBinding srcloc n1 t1 b )
>           ,Decl_Simple (SDecl_PolyBinding _      n2 t2 bs)
>           ) | n1 == n2 && t1 == t2 ->  
>                                  [Decl_Simple
>                                      (SDecl_PolyBinding srcloc n1 t1
>                                                         (b ++ bs))]

In all other cases, we do not join anything.

>         ; (_,_)              ->  [s1,s2]
>         }
>   in
>     f ++ r

We provide a variant of |groupDecls| which only groups |SDecl|s. We reuse
|groupDecls| for that by converting.

> groupSDecls                     :: [SDecl] -> [SDecl]
> groupSDecls                     =  map (\(Decl_Simple d) -> d)
>                                 .  groupDecls . map Decl_Simple 

