%% gh -- a compiler for Generic Haskell.
%% Copyright (c) 2001  The Generic Haskell Team. Utrecht University
%% 
%% This library is free software; you can redistribute it and/or modify
%% it under the terms of the GNU Lesser General Public License as
%% published by the Free Software Foundation; either version 2.1 of the
%% License, or (at your option) any later version.
%% 
%% This library is distributed in the hope that it will be useful, but
%% WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%% Lesser General Public License for more details.
%% 
%% You should have received a copy of the GNU Lesser General Public
%% License along with this library; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
%%
%% $Id: Name.lhs,v 1.5 2002/07/05 14:36:57 cvs-4 Exp $
%%
%% author: andres@cs.uu.nl

> module Name where
>
> import Char
> import Utils
> import AbstractD
> import Monad (liftM)

< import FMap

%***************************************************************************
\section{Names}
%***************************************************************************

> extractName                   :: QualName -> String
> extractName (QualName_Qual _ _ n)   = en n
> extractName (QualName_Plain _ n)    = en n
> extractName (QualName_Entity _ (Entity_Entity _ n) _) = en n
>  
> en (Name_Ident _ n)           = n
> en (Name_Symbol _ n)          = n
> en (Name_Special _ n)         = n

> getName                       :: QualName -> Name
> getName (QualName_Qual _ _ n)   =  n
> getName (QualName_Plain _ n)    =  n
> getName (QualName_Entity _ (Entity_Entity _ n) _) =  n

> isSpecial                     :: QualName -> Bool
> isSpecial                     =  isSpec . getName
>   where isSpec n = case n of { Name_Special _ _ -> True ; _ -> False }

> makeExplicitName                       :: Bool -> QualName -> QualName
> makeExplicitName False n               =  n
> makeExplicitName True (QualName_Entity p e _)= QualName_Entity p e Nothing
> makeExplicitName _ n                   =  n

> makePlainName                          :: QualName -> QualName
> makePlainName                          =  QualName_Plain unkrange . getName

The following functions allow to distinguish variables from constructors.

> isCon, isVar                  :: QualName -> Bool
> isCon                         =  isConid . extractName
> isVar                         =  isVarid . extractName

This module contains common functions used for creating the names
of the generated functions and types.

Module prefix.

> moduleName                    :: String -> String
> moduleName                    =  ("GHM_"++)

Name of the structural type.

> strName                       :: QualName -> QualName
> strName                       =  foreignDerivedConName ("GHt_"++)

Name of the structural specialisation for a type-indexed type.

> strTITName                    :: QualName -> QualName
> strTITName                    =  strName

Name of embedding projection pair.

> epName                        :: QualName -> QualName
> epName                        =  foreignDerivedVarName ("ghEP_"++)

> epTypeName,epfromName,eptoName,idEPName,liftEPName
>                               :: QualName
> epTypeName                    =  inPrelude "EP"
> epfromName                    =  inPrelude "from"
> eptoName                      =  inPrelude "to"
> idEPName                      =  inPrelude "idEP"
> liftEPName                    =  inPrelude "liftEP"

Name of bimap.

> bimapName                     :: QualName
> bimapName                     =  inPrelude "bimap"

NEW: We now distinguish component and specialisation names. Whereas
components are always located in the same module where the generic function
is declared, specialisations are always local to the current module.

> componentName                 :: QualName -> QualName -> QualName
> componentName fun ctor        =  let mangled_fun  = mangleVarQualName fun
>                                      mangled_ctor = mangleConQualName ctor
>                                      basename     = "ghc_" ++ mangleBigPair mangled_fun mangled_ctor
>                                  in  foreignDerivedVarName (const basename) fun

> specName                      :: QualName -> QualName -> QualName
> specName fun ctor             =  let mangled_fun  = mangleVarQualName fun
>                                      mangled_ctor = mangleConQualName ctor
>                                      basename     = "ghs_" ++ mangleBigPair mangled_fun mangled_ctor
>                                  in  (QualName_Plain unkrange $ Name_Ident unkrange basename)

The same for type-indexed data types.

> componentTITName              :: QualName -> QualName -> QualName
> componentTITName fun ctor     =  let mangled_fun  = mangleVarQualName fun
>                                      mangled_ctor = mangleConQualName ctor
>                                      basename     = "GHc_" ++ mangleBigPair mangled_fun mangled_ctor
>                                  in  foreignDerivedConName (const basename) fun

> specTITName                   :: QualName -> QualName -> QualName
> specTITName fun ctor          =  let mangled_fun  = mangleVarQualName fun
>                                      mangled_ctor = mangleConQualName ctor
>                                      basename     = "GHs_" ++ mangleBigPair mangled_fun mangled_ctor
>                                  in  (QualName_Plain unkrange $ Name_Ident unkrange basename)

Similar for generic abstractions.

> specGAName                    :: QualName -> Type -> QualName
> specGAName fun tp             =  let mangled_fun  = mangleVarQualName fun
>                                      mangled_tp   = mangleType tp
>                                      basename     = "ghs_" ++ mangleBigPair mangled_fun mangled_tp
>                                  in  (QualName_Plain unkrange $ Name_Ident unkrange basename)

Name of the structure type constructors (and their value constructors).

> unitName,unitunitName,conName,conconName,
>  labelName,labellabelName,sumName,sumleftName,
>  sumrightName,prodName,prodprodName,funName,listName,pairName,
>  condescrName,condescrcondescrName,labeldescrName,
>  labeldescrlabeldescrName
>                               :: QualName
> unitName                      =  inPrelude "Unit"
> unitunitName                  =  inPrelude "Unit"
> conName                       =  inPrelude "Con"
> conconName                    =  inPrelude "Con"
> labelName                     =  inPrelude "Label"
> labellabelName                =  inPrelude "Label"
> sumName                       =  inPrelude "Sum"
> sumleftName                   =  inPrelude "Inl"
> sumrightName                  =  inPrelude "Inr"
> prodName                      =  inPrelude "Prod"
> prodprodName                  =  inPreludeSymbol ":*:"
> funName                       =  inPrelude "Fun"

Generic identity function, generic identity type.

> gidName, gIDName              :: QualName
> gidName                       =  inPrelude "gid"
> gIDName                       =  inPrelude "GID"

Builtin types.

> listName                      =  hsPrelName $ Name_Special unkrange "[]"  {- was: inPrelude "LIST"   -}
> pairName                      =  hsPrelName $ Name_Special unkrange "(,)" {- was: inPrelude "TUPLE2" -}
> tupleName                     :: Int -> QualName
> tupleName n                   =  hsPrelName $ Name_Special unkrange $ "(" ++ replicate (n-1) ',' ++ ")" 
>                                     {- was: inPrelude $ "TUPLE" ++ show n -}

Constructor descriptions.

> condescrName                  =  inPrelude "ConDescr"
> condescrcondescrName          =  inPrelude "ConDescr"

Label descriptions.

> labeldescrName                =  inPrelude "LabelDescr"
> labeldescrlabeldescrName      =  inPrelude "LabelDescr"

Constructor case prefix

> concaseName                   :: QualName -> QualName
> concaseName                   =  foreignDerivedConName ("GHC_"++)

Variables in specialised types (hack).

> varTITName                    :: QualName -> QualName -> QualName
> varTITName                    =  specName

Name of constructor descriptor's fixity constructors.

> nonfixName,infixName,infixlName,infixrName
>                               :: QualName
> nonfixName                    =  inPrelude "Nonfix"
> infixName                     =  inPrelude "Infix"
> infixlName                    =  inPrelude "Infixl"
> infixrName                    =  inPrelude "Infixr"

(Real) Haskell names/basic values.

> hsPrelude                     :: String -- Modid
> hsPrelude                     =  "Prelude"
> hsPrelName                    :: Name -> QualName
> hsPrelName n                  =  QualName_Entity unkrange
>                                                  (Entity_Entity (Just hsPrelude) n) 
>                                                  (Just $ QualName_Qual unkrange hsPrelude n)
> hsUnitName                    :: QualName
> hsUnitName                    =  hsPrelName $ Name_Special unkrange "()" {- was: symbol -}
> hsFunName                     :: QualName
> hsFunName                     =  hsPrelName $ Name_Special unkrange "(->)" {- was: symbol -> -}
> hsUnitConName                 :: QualName
> hsUnitConName                 =  hsPrelName $ Name_Special unkrange "()"
> hsListName                    :: QualName
> hsListName                    =  hsPrelName $ Name_Special unkrange "[]" {- was: symbol -}
> hsTupleName                   :: Int -> QualName
> hsTupleName n                 =  hsPrelName
>                               $  Name_Special unkrange ("(" ++ replicate (n-1) ',' ++ ")") {- was: symbol -}
> hsTupleConName                :: Int -> QualName
> hsTupleConName n              =  hsPrelName 
>                               $  Name_Special unkrange ("(" ++ replicate (n-1) ',' ++ ")")
> hsMinusName                   :: Range -> Name
> hsMinusName r                 =  Name_Symbol r "-"
> hsPlingName                   :: Range -> Name
> hsPlingName r                 =  Name_Symbol r "!"
> hsDotName                     :: Range -> Name
> hsDotName r                   =  Name_Symbol r "."
> hsAsName                      :: Range -> Name
> hsAsName r                    =  Name_Ident r "as"
> hsQualifiedName               :: Range -> Name
> hsQualifiedName r             =  Name_Ident r "qualified"
> hsHidingName                  :: Range -> Name
> hsHidingName r                =  Name_Ident r "hiding"
> hsBoolName                    :: Bool -> QualName
> hsBoolName False              =  hsPrelName (Name_Ident unkrange "False")
> hsBoolName True               =  hsPrelName (Name_Ident unkrange "True")

Variable list (in cases where the name space is not dangerous).

> varNames                      :: [QualName]
> varNames                      =  map (QualName_Plain unkrange . Name_Ident unkrange) varNames'
> varNames'                     =  map (:[]) alphabet
>                               ++ [ y:x | x <- varNames', y <- alphabet ]
>   where alphabet              =  ['a'..'z']

Prefix a variable with a rare prefix to prevent name clashes.

> prefixName                    :: QualName -> QualName
> prefixName                    =  foreignDerivedVarName ("gh_"++)

Impossible name.

> impossibleName                :: QualName
> impossibleName                =  QualName_Entity unkrange 
>                                                  (Entity_Entity Nothing 
>                                                                 (Name_Special unkrange "*<impossible>")) 
>                                                  Nothing

Prefixed variable list.

> prefixedVarNames              :: [QualName]
> prefixedVarNames              =  map prefixName varNames

Name of the GH prelude.

> preludeName                   :: String -- Modid
> preludeName                   =  "GHPrelude"

Functions of the GH prelude.

> inPrelude                     :: String -> QualName
> inPrelude n                   =  QualName_Entity unkrange
>                                                  (Entity_Entity (Just preludeName) (Name_Ident unkrange n)) 
>                                                  (Just $ QualName_Qual unkrange preludeName 
>                                                                        (Name_Ident unkrange n))

> inPreludeSymbol               :: String -> QualName
> inPreludeSymbol s             =  QualName_Entity unkrange
>                                                  (Entity_Entity (Just preludeName) (Name_Symbol unkrange s)) 
>                                                  (Just $ QualName_Qual unkrange preludeName 
>                                                                        (Name_Symbol unkrange s))

E-mail address of the author.

> email                         =  "bugs@generic-haskell.org"

Derived names and mangling functions. Should be documented ;-)

> foreignDerivedConName             :: (String -> String) -> QualName -> QualName
> foreignDerivedConName f (QualName_Entity _ (Entity_Entity m n) s)
>                                   =  QualName_Entity unkrange
>                                                      (Entity_Entity (liftM foreignDerivedModuleName m) 
>                                                                     (Name_Ident unkrange $ f $ mangleConName n))
>                                                      (maybe Nothing (Just . foreignDerivedConName f) s) {- ?? -}
> foreignDerivedConName f (QualName_Plain _ n)
>                                   =  QualName_Plain unkrange (Name_Ident unkrange $ f $ mangleConName n)
> foreignDerivedConName f (QualName_Qual _ m n)
>                                   =  QualName_Qual unkrange (foreignDerivedModuleName m) 
>                                                    (Name_Ident unkrange $ f $ mangleConName n)

> foreignDerivedVarName             :: (String -> String) -> QualName -> QualName
> foreignDerivedVarName f (QualName_Entity _ (Entity_Entity m n) s)
>                                   =  QualName_Entity unkrange
>                                                      (Entity_Entity (liftM foreignDerivedModuleName m) 
>                                                                     (Name_Ident unkrange $ f $ mangleVarName n))
>                                                      (maybe Nothing (Just . foreignDerivedVarName f) s) {- ?? -}
> foreignDerivedVarName f (QualName_Plain _ n)
>                                   =  QualName_Plain unkrange (Name_Ident unkrange $ f $ mangleVarName n)
> foreignDerivedVarName f (QualName_Qual _ m n)
>                                   =  QualName_Qual unkrange (foreignDerivedModuleName m) 
>                                                    (Name_Ident unkrange $ f $ mangleVarName n)
>
> foreignDerivedModuleName          :: String -> String
> foreignDerivedModuleName m | m == hsPrelude      = preludeName
>                            | otherwise           = m

> mangleVarQualName                 :: QualName -> String
> mangleVarQualName (QualName_Plain _ n)  = mangleVarName n
> mangleVarQualName (QualName_Qual _ m n) = manglePair m (mangleVarName n)
> mangleVarQualName (QualName_Entity _ (Entity_Entity Nothing n)  _) = mangleVarName n
> mangleVarQualName (QualName_Entity _ (Entity_Entity (Just m) n) _) = manglePair m (mangleVarName n)

> mangleConQualName                 :: QualName -> String
> mangleConQualName (QualName_Plain _ n)  = mangleConName n
> mangleConQualName (QualName_Qual _ m n) = manglePair m (mangleConName n)
> mangleConQualName (QualName_Entity _ (Entity_Entity Nothing n)  _) = mangleConName n
> mangleConQualName (QualName_Entity _ (Entity_Entity (Just m) n) _) = manglePair m (mangleConName n)

> mangleVarName                     :: Name -> String
> mangleVarName (Name_Ident _ n)    =  n
> mangleVarName (Name_Symbol _ n)   =  manglePrefix "ghS_" n
> mangleVarName (Name_Special _ n)  =  manglePrefix "ghX_" n

> mangleConName                     :: Name -> String
> mangleConName (Name_Ident _ n)    =  n
> mangleConName (Name_Symbol _ n)   =  manglePrefix "GHS_" n
> mangleConName (Name_Special _ n)  =  manglePrefix "GHX_" n

> manglePrefix                      :: String -> String -> String
> manglePrefix prefix s             =  prefix ++ concatMap mangleSymbol s

> mangleBigPair            :: String -> String -> String
> mangleBigPair p1 p2      =  escape p1 ++ "__" ++ p2
>   where escape ('_':'_':r) = "q_"  ++ escape r
>         escape ('q':'_':r) = "qq_" ++ escape r
>         escape ('q':'q':r) = "qq"  ++ escape ('q':r)
>         escape (s:r)       = s : escape r
>         escape []          = []

> unmangleBigPair          :: String -> (String,String)
> unmangleBigPair          =  stripFirst ""
>   where stripFirst acc ('_':'_':r) = (reverse acc,r)
>         stripFirst acc ('q':'q':r) = stripFirst ('q':acc) r
>         stripFirst acc ('q':'_':r) = stripFirst ("__" ++ acc) r
>         stripFirst acc (n      :r) = stripFirst (n:acc) r
>         stripFirst acc []          = (reverse acc,"") -- error tolerance

> manglePair               :: String -> String -> String
> manglePair p1 p2         =  escape p1 ++ "_" ++ p2
>   where escape           =  concatMap escapeChar
>         escapeChar '_'   =  "q_"
>         escapeChar 'q'   =  "qq"
>         escapeChar  x    =  [x]

> unmanglePair             :: String -> (String,String)
> unmanglePair             =  stripFirst ""
>   where stripFirst acc ('_' : r) = (reverse acc,r)
>         stripFirst acc []        = (reverse acc,"")  -- error tolerance
>         stripFirst acc ('q':n:r) = stripFirst (n:acc) r
>         stripFirst acc (n   : r) = stripFirst (n:acc) r

> mangleList               :: [String] -> String
> mangleList               =  foldr1 manglePair

> unmangleList             :: Int -> String -> [String]
> unmangleList 1 p         =  [p]
> unmangleList n p         =  let (p1,p2) = unmanglePair p
>                             in  p1 : unmangleList (n-1) p2

> mangleSymbol             :: Char -> String
> mangleSymbol '&'         =  "ad"
> mangleSymbol '$'         =  "ap"
> mangleSymbol '@'         =  "at"
> mangleSymbol ']'         =  "bc"
> mangleSymbol '\\'        =  "bk"
> mangleSymbol '['         =  "bo"
> mangleSymbol '`'         =  "bq"
> mangleSymbol '}'         =  "cc"
> mangleSymbol ':'         =  "cl"
> mangleSymbol ','         =  "cm"
> mangleSymbol '{'         =  "co"
> mangleSymbol '.'         =  "dt"
> mangleSymbol '/'         =  "dv"
> mangleSymbol '='         =  "eq"
> mangleSymbol '!'         =  "ex"
> mangleSymbol '>'         =  "gt"
> mangleSymbol '#'         =  "hs"
> mangleSymbol '<'         =  "lt"
> mangleSymbol '%'         =  "md"
> mangleSymbol '-'         =  "mn"
> mangleSymbol '?'         =  "op"
> mangleSymbol '|'         =  "or"
> mangleSymbol ')'         =  "pc"
> mangleSymbol '+'         =  "pl"
> mangleSymbol '('         =  "po"
> mangleSymbol '^'         =  "pw"
> mangleSymbol '"'         =  "qu"
> mangleSymbol ';'         =  "sc"
> mangleSymbol '~'         =  "tl"
> mangleSymbol '*'         =  "tm"
> mangleSymbol c           =  "N" ++ [c]

> unmangle                 :: String -> Range -> Name
> unmangle ('G':'H':'S':'_':rest) r = Name_Symbol r $ unmangle' rest
> unmangle ('g':'h':'S':'_':rest) r = Name_Symbol r $ unmangle' rest
> unmangle ('G':'H':'X':'_':rest) r = Name_Special r $ unmangle' rest
> unmangle ('g':'h':'X':'_':rest) r = Name_Special r $ unmangle' rest
> unmangle x r             =  Name_Ident r x
>
> unmangle' []             =  []
> unmangle' [x]            =  [x]
> unmangle' (x:y:rest)     = unmangleSymbol [x,y] : unmangle' rest
>
> unmangleSymbol           :: String -> Char
> unmangleSymbol "ad"      =  '&'
> unmangleSymbol "ap"      =  '$'
> unmangleSymbol "at"      =  '@'
> unmangleSymbol "bc"      =  ']'
> unmangleSymbol "bk"      =  '\\'
> unmangleSymbol "bo"      =  '['
> unmangleSymbol "bq"      =  '`'
> unmangleSymbol "cc"      =  '}'
> unmangleSymbol "cl"      =  ':'
> unmangleSymbol "cm"      =  ','
> unmangleSymbol "co"      =  '{'
> unmangleSymbol "dt"      =  '.'
> unmangleSymbol "dv"      =  '/'
> unmangleSymbol "eq"      =  '='
> unmangleSymbol "ex"      =  '!'
> unmangleSymbol "gt"      =  '>'
> unmangleSymbol "hs"      =  '#'
> unmangleSymbol "lt"      =  '<'
> unmangleSymbol "md"      =  '%'
> unmangleSymbol "mn"      =  '-'
> unmangleSymbol "op"      =  '?'
> unmangleSymbol "or"      =  '|'
> unmangleSymbol "pc"      =  ')'
> unmangleSymbol "pl"      =  '+'
> unmangleSymbol "po"      =  '('
> unmangleSymbol "pw"      =  '^'
> unmangleSymbol "qu"      =  '"'
> unmangleSymbol "sc"      =  ';'
> unmangleSymbol "tl"      =  '~'
> unmangleSymbol "tm"      =  '*'

> mangleType               :: Type -> String
> mangleType t             =  "GHT_" ++ mangleType' t
>   where mangleType'                     :: Type -> String
>         mangleType' (Type_App l r)      =  mangleList ["a",mangleType' l,mangleType' r]
>         mangleType' (Type_Lam f b)      =  mangleList ["l",mangleVarQualName f,mangleType' b]
>         mangleType' (Type_All v b)      =  mangleList ["A",mangleVarQualName v,mangleType' b]
>         mangleType' (Type_Var _ n)      =  mangleList ["v",mangleVarQualName n]
>         mangleType' (Type_Con _ n)      =  mangleList ["c",mangleConQualName n]
>         mangleType' Type_Unit           =  "U"
>         mangleType' Type_List           =  "L"
>         mangleType' Type_Fun            =  "F"
>         mangleType' (Type_Tuple n)      =  "T" ++ show n
>         mangleType' (Type_KindApp _ _ _)=  "<error:kindapp>"
>         mangleType' (Type_TypeApp _ _ _)=  "<error:typeapp>"
>         mangleType' (Type_ConCase _ n)  =  "C" ++ mangleConQualName n

Does not really belong here, but it is the easiest solution for now:

> stype_ctor                     :: SType -> QualName
> stype_ctor (SType_Const _ n)   =  n
> stype_ctor SType_Sum           =  sumName
> stype_ctor SType_Prod          =  prodName
> stype_ctor (SType_Con _)       =  conName
> stype_ctor (SType_Label _)     =  labelName
> stype_ctor SType_Unit          =  unitName
> stype_ctor (SType_Var _ n)     =  n {- should be: internalError "Components.stype_ctor: Var case called" -}
> stype_ctor (SType_ConCase _ n) =  concaseName n
> stype_ctor SType_Fun           =  funName

> instance Eq SType where
>   st1 == st2 = stype_ctor st1 == stype_ctor st2
