-- gh -- a compiler for Generic Haskell.
-- Copyright (c) 2001  The Generic Haskell Team. Utrecht University
-- 
-- This library is free software; you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as
-- published by the Free Software Foundation; either version 2.1 of the
-- License, or (at your option) any later version.
-- 
-- This library is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-- Lesser General Public License for more details.
-- 
-- You should have received a copy of the GNU Lesser General Public
-- License along with this library; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
--
-- $Id: Pretty.hs,v 1.3 2002/07/04 12:58:47 cvs-4 Exp $
--
-- author: Andres Loeh (andres@cs.uu.nl)

module Pretty ( module UU_Pretty, module UU_Pretty_ext
              , pp_ite_sp, pp_tbraces, pp_kbraces, pretty, pp_align
              , ppPar, ppFunApp, ppInfixApp, ppForall
              , ppLet, ppCase, ppIf, ppDo, ppWhere
              , ppStep, ppSym
              , ppTupleBlock, ppListBlock, ppRecordBlock
              , isEmpty
              ) where

import UU_Pretty
import UU_Pretty_ext

-- the same as pp_ite, but with better spacing and without a fi symbol
pp_ite_sp                       :: (PP a, PP b, PP c, PP d, PP e, PP f)
                                => a -> b -> c
                                -> d -> e -> f
                                -> PP_Doc
pp_ite_sp kw_if kw_then kw_else 
          c t e                 = (     eelement_h1 ( par >>#<< par >>#<< par )
				  >>^<< (     (     ( par >>#<< par >>^<< par >>-<< par )
					      >>$<< [par, par >>-<< par]
					      )
					)
				  )  >>$< [ kw_if   >#< c
					  , kw_then >#< t
					  , kw_else >#< e
					  ]

-- special braces for types in generic functions
pp_kbraces                      =  pp_wrap "{[ " " ]}"
pp_tbraces                      =  pp_wrap "{| " " |}"

-- pretty print a (single layout) document with a very hight line width

pretty                          :: PP a => a -> String
pretty x                        =  disp (pp x) maxBound ""

-- pretty print a document aligned on a middle column

pp_align                        :: (PP a,PP b) => a -> b -> PP_Doc
pp_align lt rt                  =  
  let ltp                       =  pretty lt
      lll                       =  length . last . lines $ ltp
      col                       =  31    -- hard-coded align column
  in  if   lll >= col 
      then ltp >-< indent col rt
      else ltp >|< indent (col - lll) rt

ppPar c                         =  if c then pp_parens else id

ppFunApp                        :: Int -> [PP_Doc] -> PP_Doc -> [PP_Doc]
                                -> PP_Doc
ppFunApp prec []  fun []        =  fun
ppFunApp prec []  fun args      =  ppPar (prec > 8)
                                         (fun >#< hv_sp args)
ppFunApp prec all fun args      =  ppForall all >|< fun >#< hv_sp args

ppInfixApp                      :: Int -> [PP_Doc] -> PP_Doc -> [PP_Doc]
                                -> PP_Doc
ppInfixApp prec [] fun [a1,a2]  =  ppPar (prec > 8) (a1 >#< fun >#< a2)
ppInfixApp prec all fun [a1,a2] =  ppForall all >|< a1 >#< fun >#< a2

ppForall                        :: [PP_Doc] -> PP_Doc
ppForall []                     =  empty
ppForall all                    =  "forall" >#< hlist_sp (reverse all) 
                                            >|< ". "
ppLet :: Bool -> [PP_Doc] -> PP_Doc -> PP_Doc
ppLet i ds b = (     (par >>#<< par >>-<< par >>#<< par)
               >>^<< (     par 
                     >>-<< eindent 1 par
                     >>-<< par
                     >>-<< eindent 1 par
                     )
               ) >>$< [pp "let",llist i ds,pp "in ",b]

ppCase :: Bool -> PP_Doc -> [PP_Doc] -> PP_Doc
ppCase i e alts = (     (par >>#<< par >>#<< par >>-<< eindent 1 par)
                  >>^<< (     par
                        >>-<< eindent 1 par
                        >>-<< par
                        >>-<< eindent 1 par
                        )
                  ) >>$< [pp "case",e,pp "of",llist i alts]

ppIf :: PP_Doc -> PP_Doc -> PP_Doc -> PP_Doc
ppIf c t e = pp_ite_sp "if" "then" "else" c t e

ppDo :: Bool -> [PP_Doc] -> PP_Doc
ppDo i stmts = ppStep "do" (llist i stmts)

llist :: Bool -> [PP_Doc] -> PP_Doc
llist False stmts = vlist stmts
llist True  stmts = vblock stmts

vblock :: [PP_Doc] -> PP_Doc
vblock [] = text "{ }"
vblock (x : xs) = vlist (("{" >#< x) : map (";" >#<) xs ++ [text "}"])

ppWhere :: PP_Doc -> [PP_Doc] -> PP_Doc
ppWhere body [] = body
ppWhere body w  = body >-< "where" >#< vlist w

ppStep :: (PP a,PP b) => a -> b -> PP_Doc
ppStep l r =  (     (par >>#<< par) 
              >>^<< (par >>-<< eindent 1 par)
              ) >>$< [pp l,pp r]

ppSym :: PP a => PP_Doc -> a -> PP_Doc -> PP_Doc
ppSym l s r = ppStep l (s >#< r)

ppTupleBlock,ppListBlock,ppRecordBlock :: [PP_Doc] -> PP_Doc
ppTupleBlock =  pp_block "(" ")" ","
ppListBlock  =  pp_block "[" "]" ","
ppRecordBlock=  pp_block "{" "}" ","
ppStatBlock  =  pp_block "{" "}" ";"

isEmpty :: PP a => a -> Bool
isEmpty =  null . pretty
