%% gh -- a compiler for Generic Haskell.
%% Copyright (c) 2001  The Generic Haskell Team. Utrecht University
%% 
%% This library is free software; you can redistribute it and/or modify
%% it under the terms of the GNU Lesser General Public License as
%% published by the Free Software Foundation; either version 2.1 of the
%% License, or (at your option) any later version.
%% 
%% This library is distributed in the hope that it will be useful, but
%% WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%% Lesser General Public License for more details.
%% 
%% You should have received a copy of the GNU Lesser General Public
%% License along with this library; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
%%
%% $Id: Range.lhs,v 1.2 2002/04/15 11:56:01 cvs-4 Exp $
%%
%% author: Andres Loeh (andres@cs.uu.nl)

%***************************************************************************
\section{Ranges}
%***************************************************************************

\begin{code}
module Range where

import AbstractD
\end{code}

%---------------------------------------------------------------------------
\subsection{Class definition and general derived functions}
%---------------------------------------------------------------------------

\begin{code}

class HasRange a where
  getRange :: a -> Range
  setRange :: a -> Range -> a

rangeFromTo                     :: (HasRange a,HasRange b) => a -> b -> Range
rangeFromTo x y                 =  case (getRange x,getRange y) of
                                     (Range_Range f _,Range_Range _ t) -> 
                                        Range_Range f t

\end{code}

%---------------------------------------------------------------------------
\subsection{Instances}
%---------------------------------------------------------------------------

\begin{code}

instance HasRange Name where
  getRange (Name_Ident   r _) = r
  getRange (Name_Symbol  r _) = r
  getRange (Name_Special r _) = r
  setRange (Name_Ident   _ n) r = Name_Ident   r n
  setRange (Name_Symbol  _ n) r = Name_Symbol  r n
  setRange (Name_Special _ n) r = Name_Special r n

instance HasRange QualName where
  getRange (QualName_Qual   r _ _) = r
  getRange (QualName_Plain  r _)   = r
  getRange (QualName_Entity r _ _) = r
  setRange (QualName_Qual   _ m n) r = QualName_Qual   r m n
  setRange (QualName_Plain  _ n)   r = QualName_Plain  r n
  setRange (QualName_Entity _ e s) r = QualName_Entity r e s

\end{code}
