%% gh -- a compiler for Generic Haskell.
%% Copyright (c) 2001  The Generic Haskell Team. Utrecht University
%% 
%% This library is free software; you can redistribute it and/or modify
%% it under the terms of the GNU Lesser General Public License as
%% published by the Free Software Foundation; either version 2.1 of the
%% License, or (at your option) any later version.
%% 
%% This library is distributed in the hope that it will be useful, but
%% WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%% Lesser General Public License for more details.
%% 
%% You should have received a copy of the GNU Lesser General Public
%% License along with this library; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
%%
%% $Id: TiTypes.lhs,v 1.3 2002/05/24 16:01:27 cvs-4 Exp $
%%
%% author: Andres Loeh (andres@cs.uu.nl)

%***************************************************************************
\section{Type-indexed types}
%***************************************************************************

\begin{code}
module TiTypes where

import Name
import Utils
import ErrorUtils
import AbstractD
import FMap
\end{code}

This module provides functions to handle type-indexed types.
The function |handleTiTypes| spots type applications in a type
an resolves them: applications to complex types are replaced by
multiple type applications; applications to constant types are
replaced by the specialisations; finally, applications to variables
are resolved by making the entire type application a type variable.

\begin{code}

handleTiTypesQ                 :: FMap QualName QualNames -> QualType -> QualType
handleTiTypesQ deps (QualType_QualType ctx tp) = QualType_QualType ctx (handleTiTypes deps tp)
  {- context should also be handled -}

handleTiTypes                  :: FMap QualName QualNames -> Type -> Type
handleTiTypes deps t           =  case t of
  Type_App left right          -> Type_App (handleTiTypes deps left)
                                           (handleTiTypes deps right)
  Type_Lam from body           -> Type_Lam from (handleTiTypes deps body)
  Type_All var body            -> Type_All var (handleTiTypes deps body)
  Type_TypeApp _ var tp        -> typeApply deps var tp
  x                            -> x

typeApply                      :: FMap QualName QualNames -> QualName -> Type -> Type
typeApply deps v t             =  case t of
  Type_App left right          -> tapply' (typeApply deps v left)
                                          (map (\ x -> typeApply deps x right) dgfs)
  Type_Lam from body           -> typeApplyLambdaError
  Type_All from body           -> typeApplyAllError
  Type_Var sl name             -> Type_Var sl $ varTITName name v
  Type_Con sl name             -> Type_Con sl $ specTITName v name
  Type_Unit                    -> Type_Con genloc $ specTITName v unitName
  Type_List                    -> Type_Con genloc $ specTITName v listName
  Type_Fun                     -> Type_Con genloc $ specTITName v funName
  Type_Tuple n                 -> Type_Con genloc $ specTITName v (tupleName n)
  Type_KindApp _ var kind      -> typeApplyKindAppError
  Type_TypeApp _ var tp        -> typeApply deps v (typeApply deps var tp)
 where
  dgfs                         =  maybe [v] id (find deps v)

typeApplyLambdaError
  = unstaticError "lambda occurs in type application"

typeApplyAllError
  = unstaticError "forall occurs in type application"

typeApplyKindAppError
  = internalError "kindapps should be removed prior to typeapps"

\end{code}
